@@ -1,31 +1,16 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Command dispatcher for TabbedBrowser."""
 
 import os.path
 import shlex
 import functools
-from typing import cast, Callable, Dict, Union
+from typing import cast, Callable, Dict, Union, Optional
 
-from PyQt5.QtWidgets import QApplication, QTabBar
-from PyQt5.QtCore import pyqtSlot, Qt, QUrl, QEvent, QUrlQuery
+from qutebrowser.qt.widgets import QApplication, QTabBar
+from qutebrowser.qt.core import Qt, QUrl, QEvent, QUrlQuery
 
 from qutebrowser.commands import userscripts, runners
 from qutebrowser.api import cmdutils
@@ -70,9 +55,7 @@ class CommandDispatcher:
             raise cmdutils.CommandError("Private windows are unavailable with "
                                         "the single-process process model.")
 
-        new_window = mainwindow.MainWindow(private=private)
-        new_window.show()
-        return new_window.tabbed_browser
+        return mainwindow.MainWindow(private=private).tabbed_browser
 
     def _count(self) -> int:
         """Convenience method to get the widget count."""
@@ -109,8 +92,15 @@ class CommandDispatcher:
             raise cmdutils.CommandError("No WebView available yet!")
         return widget
 
-    def _open(self, url, tab=False, background=False, window=False,
-              related=False, private=None):
+    def _open(
+        self,
+        url: QUrl,
+        tab: bool = False,
+        background: bool = False,
+        window: bool = False,
+        related: bool = False,
+        private: Optional[bool] = None,
+    ) -> None:
         """Helper function to open a page.
 
         Args:
@@ -123,13 +113,15 @@ class CommandDispatcher:
         """
         urlutils.raise_cmdexc_if_invalid(url)
         tabbed_browser = self._tabbed_browser
-        cmdutils.check_exclusive((tab, background, window, private), 'tbwp')
+        cmdutils.check_exclusive((tab, background, window, private or False), 'tbwp')
         if window and private is None:
             private = self._tabbed_browser.is_private
 
         if window or private:
+            assert isinstance(private, bool)
             tabbed_browser = self._new_tabbed_browser(private)
             tabbed_browser.tabopen(url)
+            tabbed_browser.window().show()
         elif tab:
             tabbed_browser.tabopen(url, background=False, related=related)
         elif background:
@@ -192,21 +184,21 @@ class CommandDispatcher:
                       what's configured in 'tabs.select_on_remove'.
 
         Return:
-            QTabBar.SelectLeftTab, QTabBar.SelectRightTab, or None if no change
+            QTabBar.SelectionBehavior.SelectLeftTab, QTabBar.SelectionBehavior.SelectRightTab, or None if no change
             should be made.
         """
         cmdutils.check_exclusive((prev, next_, opposite), 'pno')
         if prev:
-            return QTabBar.SelectLeftTab
+            return QTabBar.SelectionBehavior.SelectLeftTab
         elif next_:
-            return QTabBar.SelectRightTab
+            return QTabBar.SelectionBehavior.SelectRightTab
         elif opposite:
             conf_selection = config.val.tabs.select_on_remove
-            if conf_selection == QTabBar.SelectLeftTab:
-                return QTabBar.SelectRightTab
-            elif conf_selection == QTabBar.SelectRightTab:
-                return QTabBar.SelectLeftTab
-            elif conf_selection == QTabBar.SelectPreviousTab:
+            if conf_selection == QTabBar.SelectionBehavior.SelectLeftTab:
+                return QTabBar.SelectionBehavior.SelectRightTab
+            elif conf_selection == QTabBar.SelectionBehavior.SelectRightTab:
+                return QTabBar.SelectionBehavior.SelectLeftTab
+            elif conf_selection == QTabBar.SelectionBehavior.SelectPreviousTab:
                 raise cmdutils.CommandError(
                     "-o is not supported with 'tabs.select_on_remove' set to "
                     "'last-used'!")
@@ -383,17 +375,18 @@ class CommandDispatcher:
                 yield parsed
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
-    def tab_clone(self, bg=False, window=False):
+    def tab_clone(self, bg=False, window=False, private=False):
         """Duplicate the current tab.
 
         Args:
             bg: Open in a background tab.
             window: Open in a new window.
+            private: Open in a new private window.
 
         Return:
             The new QWebView.
         """
-        cmdutils.check_exclusive((bg, window), 'bw')
+        cmdutils.check_exclusive((bg, window, private), 'bwp')
         curtab = self._current_widget()
         cur_title = self._tabbed_browser.widget.page_title(
             self._current_index())
@@ -402,14 +395,17 @@ class CommandDispatcher:
         except browsertab.WebTabError as e:
             raise cmdutils.CommandError(e)
 
-        # The new tab could be in a new tabbed_browser (e.g. because of
-        # tabs.tabs_are_windows being set)
-        if window:
+        if window or private:
             new_tabbed_browser = self._new_tabbed_browser(
-                private=self._tabbed_browser.is_private)
+                private=self._tabbed_browser.is_private or private)
         else:
             new_tabbed_browser = self._tabbed_browser
+
         newtab = new_tabbed_browser.tabopen(background=bg)
+        new_tabbed_browser.window().show()
+
+        # The new tab could be in a new tabbed_browser (e.g. because of
+        # tabs.tabs_are_windows being set)
         new_tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                         window=newtab.win_id)
         idx = new_tabbed_browser.widget.indexOf(newtab)
@@ -450,7 +446,7 @@ class CommandDispatcher:
 
         self._open(tab.url(), tab=True)
         if not keep:
-            tabbed_browser.close_tab(tab, add_undo=False)
+            tabbed_browser.close_tab(tab, add_undo=False, transfer=True)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('win_id', completion=miscmodels.window)
@@ -497,18 +493,38 @@ class CommandDispatcher:
                     "The window with id {} is not private".format(win_id))
 
         tabbed_browser.tabopen(self._current_url())
+        tabbed_browser.window().show()
+
         if not keep:
             self._tabbed_browser.close_tab(self._current_widget(),
-                                           add_undo=False)
+                                           add_undo=False,
+                                           transfer=True)
 
-    def _back_forward(self, tab, bg, window, count, forward, index=None):
+    def _back_forward(
+        self, *,
+        tab: bool,
+        bg: bool,
+        window: bool,
+        count: Optional[int],
+        forward: bool,
+        quiet: bool,
+        index: Optional[int],
+    ) -> None:
         """Helper function for :back/:forward."""
         history = self._current_widget().history
-        # Catch common cases before e.g. cloning tab
+        # Catch common cases before e.g. cloning tab, and handle --quiet
         if not forward and not history.can_go_back():
-            raise cmdutils.CommandError("At beginning of history.")
+            text = "At beginning of history."
+            if quiet:
+                log.webview.debug(text)
+                return
+            raise cmdutils.CommandError(text)
         if forward and not history.can_go_forward():
-            raise cmdutils.CommandError("At end of history.")
+            text = "At end of history."
+            if quiet:
+                log.webview.debug(text)
+                return
+            raise cmdutils.CommandError(text)
 
         if tab or bg or window:
             widget = self.tab_clone(bg, window)
@@ -534,7 +550,7 @@ class CommandDispatcher:
     @cmdutils.argument('index', completion=miscmodels.back)
     def back(self, tab: bool = False, bg: bool = False,
              window: bool = False, count: int = None,
-             index: int = None) -> None:
+             index: int = None, quiet: bool = False) -> None:
         """Go back in the history of the current tab.
 
         Args:
@@ -543,15 +559,24 @@ class CommandDispatcher:
             window: Go back in a new window.
             count: How many pages to go back.
             index: Which page to go back to, count takes precedence.
+            quiet: Don't show an error if already at the beginning of history.
         """
-        self._back_forward(tab, bg, window, count, forward=False, index=index)
+        self._back_forward(
+            tab=tab,
+            bg=bg,
+            window=window,
+            count=count,
+            forward=False,
+            index=index,
+            quiet=quiet,
+        )
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('count', value=cmdutils.Value.count)
     @cmdutils.argument('index', completion=miscmodels.forward)
     def forward(self, tab: bool = False, bg: bool = False,
                 window: bool = False, count: int = None,
-                index: int = None) -> None:
+                index: int = None, quiet: bool = False) -> None:
         """Go forward in the history of the current tab.
 
         Args:
@@ -560,8 +585,17 @@ class CommandDispatcher:
             window: Go forward in a new window.
             count: How many pages to go forward.
             index: Which page to go forward to, count takes precedence.
+            quiet: Don't show an error if already at the end of history.
         """
-        self._back_forward(tab, bg, window, count, forward=True, index=index)
+        self._back_forward(
+            tab=tab,
+            bg=bg,
+            window=window,
+            count=count,
+            forward=True,
+            index=index,
+            quiet=quiet,
+        )
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('where', choices=['prev', 'next', 'up', 'increment',
@@ -602,7 +636,7 @@ class CommandDispatcher:
         widget = self._current_widget()
         url = self._current_url()
 
-        handlers: Dict[str, Callable] = {
+        handlers: Dict[str, Callable[..., QUrl]] = {
             'prev': functools.partial(navigate.prevnext, prev=True),
             'next': functools.partial(navigate.prevnext, prev=False),
             'up': navigate.path_up,
@@ -669,9 +703,10 @@ class CommandDispatcher:
         assert what in ['url', 'pretty-url'], what
 
         if what == 'pretty-url':
-            flags = QUrl.RemovePassword | QUrl.DecodeReserved
+            flags = urlutils.FormatOption.DECODE_RESERVED
         else:
-            flags = QUrl.RemovePassword | QUrl.FullyEncoded
+            flags = urlutils.FormatOption.ENCODED
+        flags |= urlutils.FormatOption.REMOVE_PASSWORD
 
         url = QUrl(self._current_url())
         url_query = QUrlQuery()
@@ -683,7 +718,7 @@ class CommandDispatcher:
             if key in config.val.url.yank_ignored_parameters:
                 url_query.removeQueryItem(key)
         url.setQuery(url_query)
-        return url.toString(flags)  # type: ignore[arg-type]
+        return url.toString(flags)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('what', choices=['selection', 'url', 'pretty-url',
@@ -756,12 +791,17 @@ class CommandDispatcher:
                               maybe=True)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
-    def tab_only(self, prev=False, next_=False, force=False):
+    @cmdutils.argument('pinned', choices=['prompt', 'close', 'keep'], flag='P',
+                       metavar='behavior')
+    @cmdutils.argument('force', hide=True)
+    def tab_only(self, *, prev=False, next_=False, pinned='prompt', force=False):
         """Close all tabs except for the current one.
 
         Args:
             prev: Keep tabs before the current.
             next_: Keep tabs after the current.
+            pinned: What to do with pinned tabs.
+                    Valid values: prompt, close, keep.
             force: Avoid confirmation for pinned tabs.
         """
         cmdutils.check_exclusive((prev, next_), 'pn')
@@ -775,24 +815,28 @@ class CommandDispatcher:
                         (prev and i < cur_idx) or
                         (next_ and i > cur_idx))
 
+        if force:
+            message.warning("--force is deprecated, use --pinned close instead.")
+            pinned = 'close'
+
         # close as many tabs as we can
         first_tab = True
         pinned_tabs_cleanup = False
         for i, tab in enumerate(self._tabbed_browser.widgets()):
             if _to_close(i):
-                if force or not tab.data.pinned:
+                if pinned == 'close' or not tab.data.pinned:
                     self._tabbed_browser.close_tab(tab, new_undo=first_tab)
                     first_tab = False
                 else:
                     pinned_tabs_cleanup = tab
 
         # Check to see if we would like to close any pinned tabs
-        if pinned_tabs_cleanup:
+        if pinned_tabs_cleanup and pinned == 'prompt':
             self._tabbed_browser.tab_close_prompt_if_pinned(
                 pinned_tabs_cleanup,
-                force,
+                pinned == 'close',
                 lambda: self.tab_only(
-                    prev=prev, next_=next_, force=True),
+                    prev=prev, next_=next_, pinned='close'),
                 text="Are you sure you want to close pinned tabs?")
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
@@ -902,6 +946,7 @@ class CommandDispatcher:
                 # Not sure how you enter a command without an active window...
                 raise cmdutils.CommandError(
                     "No window specified and couldn't find active window!")
+            assert isinstance(active_win, mainwindow.MainWindow), active_win
             win_id = active_win.win_id
 
         if win_id not in objreg.window_registry:
@@ -943,8 +988,7 @@ class CommandDispatcher:
         tabbed_browser, tab = self._resolve_tab_index(index)
 
         window = tabbed_browser.widget.window()
-        window.activateWindow()
-        window.raise_()
+        mainwindow.raise_window(window)
         tabbed_browser.widget.setCurrentWidget(tab)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
@@ -974,6 +1018,9 @@ class CommandDispatcher:
             self._tab_focus_stack(index)
             return
         elif index is None:
+            message.warning(
+                "Using :tab-focus without count is deprecated, "
+                "use :tab-next instead.")
             self.tab_next()
             return
 
@@ -992,11 +1039,10 @@ class CommandDispatcher:
             raise cmdutils.CommandError("There's no tab with index {}!".format(
                 index))
 
-    @cmdutils.register(instance='command-dispatcher', scope='window')
-    @cmdutils.argument('index', choices=['+', '-'])
-    @cmdutils.argument('count', value=cmdutils.Value.count)
-    def tab_move(self, index: Union[str, int] = None,
-                 count: int = None) -> None:
+    @cmdutils.register(instance="command-dispatcher", scope="window")
+    @cmdutils.argument("index", choices=["+", "-", "start", "end"])
+    @cmdutils.argument("count", value=cmdutils.Value.count)
+    def tab_move(self, index: Union[str, int] = None, count: int = None) -> None:
         """Move the current tab according to the argument and [count].
 
         If neither is given, move it to the first position.
@@ -1005,24 +1051,30 @@ class CommandDispatcher:
             index: `+` or `-` to move relative to the current tab by
                    count, or a default of 1 space.
                    A tab index to move to that index.
+                   `start` and `end` to move to the start and the end.
             count: If moving relatively: Offset.
                    If moving absolutely: New position (default: 0). This
                    overrides the index argument, if given.
         """
-        if index in ['+', '-']:
+        if index in ["+", "-"]:
             # relative moving
             new_idx = self._current_index()
             delta = 1 if count is None else count
-            if index == '-':
+            if index == "-":
                 new_idx -= delta
-            elif index == '+':  # pragma: no branch
+            elif index == "+":  # pragma: no branch
                 new_idx += delta
 
             if config.val.tabs.wrap:
                 new_idx %= self._count()
         else:
+            # pylint: disable=else-if-used
             # absolute moving
-            if count is not None:
+            if index == "start":
+                new_idx = 0
+            elif index == "end":
+                new_idx = self._count() - 1
+            elif count is not None:
                 new_idx = count - 1
             elif index is not None:
                 assert isinstance(index, int)
@@ -1077,12 +1129,11 @@ class CommandDispatcher:
         log.procs.debug("Executing {} with args {}, userscript={}".format(
             cmd, args, userscript))
 
-        @pyqtSlot()
-        def _on_proc_finished():
+        def _on_proc_finished(proc):
             if output:
                 tb = objreg.get('tabbed-browser', scope='window',
                                 window='last-focused')
-                tb.load_url(QUrl('qute://spawn-output'), newtab=True)
+                tb.load_url(QUrl(f'qute://process/{proc.pid}'), newtab=True)
 
         if userscript:
             def _selection_callback(s):
@@ -1112,7 +1163,7 @@ class CommandDispatcher:
                                  "detailed error")
             else:
                 proc.start(cmd, args)
-            proc.finished.connect(_on_proc_finished)
+            proc.finished.connect(functools.partial(_on_proc_finished, proc))
 
     def _run_userscript(self, selection, cmd, args, verbose, output_messages,
                         count):
@@ -1135,17 +1186,19 @@ class CommandDispatcher:
 
         idx = self._current_index()
         if idx != -1:
+            env['QUTE_TAB_INDEX'] = str(idx + 1)
             env['QUTE_TITLE'] = self._tabbed_browser.widget.page_title(idx)
 
-        # FIXME:qtwebengine: If tab is None, run_async will fail!
         tab = self._tabbed_browser.widget.currentWidget()
+        if tab is None:
+            raise cmdutils.CommandError("No current tab!")
 
         try:
             url = self._tabbed_browser.current_url()
         except qtutils.QtValueError:
             pass
         else:
-            env['QUTE_URL'] = url.toString(QUrl.FullyEncoded)
+            env['QUTE_URL'] = url.toString(QUrl.ComponentFormattingOption.FullyEncoded)
 
         try:
             runner = userscripts.run_async(
@@ -1182,21 +1235,31 @@ class CommandDispatcher:
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        maxsplit=0)
     @cmdutils.argument('name', completion=miscmodels.quickmark)
-    def quickmark_del(self, name=None):
+    def quickmark_del(self, name=None, all_=False):
         """Delete a quickmark.
 
         Args:
             name: The name of the quickmark to delete. If not given, delete the
                   quickmark for the current page (choosing one arbitrarily
                   if there are more than one).
+            all_: Delete all quickmarks.
         """
         quickmark_manager = objreg.get('quickmark-manager')
+
+        if all_:
+            if name is not None:
+                raise cmdutils.CommandError("Cannot specify name and --all")
+            quickmark_manager.clear()
+            message.info("Quickmarks cleared.")
+            return
+
         if name is None:
             url = self._current_url()
             try:
                 name = quickmark_manager.get_by_qurl(url)
             except urlmarks.DoesNotExistError as e:
                 raise cmdutils.CommandError(str(e))
+
         try:
             quickmark_manager.delete(name)
         except KeyError:
@@ -1239,9 +1302,8 @@ class CommandDispatcher:
             was_added = bookmark_manager.add(url, title, toggle=toggle)
         except urlmarks.Error as e:
             raise cmdutils.CommandError(str(e))
-        else:
-            msg = "Bookmarked {}" if was_added else "Removed bookmark {}"
-            message.info(msg.format(url.toDisplayString()))
+        msg = "Bookmarked {}" if was_added else "Removed bookmark {}"
+        message.info(msg.format(url.toDisplayString()))
 
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        maxsplit=0)
@@ -1268,18 +1330,28 @@ class CommandDispatcher:
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        maxsplit=0)
     @cmdutils.argument('url', completion=miscmodels.bookmark)
-    def bookmark_del(self, url=None):
+    def bookmark_del(self, url=None, all_=False):
         """Delete a bookmark.
 
         Args:
             url: The url of the bookmark to delete. If not given, use the
                  current page's url.
+            all_: If given, delete all bookmarks.
         """
+        bookmark_manager = objreg.get('bookmark-manager')
+        if all_:
+            if url is not None:
+                raise cmdutils.CommandError("Cannot specify url and --all")
+            bookmark_manager.clear()
+            message.info("Bookmarks cleared.")
+            return
+
         if url is None:
-            url = self._current_url().toString(QUrl.RemovePassword |
-                                               QUrl.FullyEncoded)
+            url = self._current_url().toString(QUrl.UrlFormattingOption.RemovePassword |
+                                               QUrl.ComponentFormattingOption.FullyEncoded)
+
         try:
-            objreg.get('bookmark-manager').delete(url)
+            bookmark_manager.delete(url)
         except KeyError:
             raise cmdutils.CommandError("Bookmark '{}' not found!".format(url))
         message.info("Removed bookmark {}".format(url))
@@ -1504,6 +1576,7 @@ class CommandDispatcher:
         Callback for GUIProcess when the edited text was updated.
 
         Args:
+            ed: The editor.ExternalEditor instance
             elem: The WebElementWrapper which was modified.
             text: The new text to insert.
         """
@@ -1518,32 +1591,37 @@ class CommandDispatcher:
             message.error(str(e))
             ed.backup()
 
-    def _search_cb(self, found, *, tab, old_scroll_pos, options, text, prev):
-        """Callback called from search/search_next/search_prev.
+    def _search_cb(self, found, *, text):
+        """Callback called from :search.
 
         Args:
             found: Whether the text was found.
-            tab: The AbstractTab in which the search was made.
-            old_scroll_pos: The scroll position (QPoint) before the search.
-            options: The options (dict) the search was made with.
-            text: The text searched for.
-            prev: Whether we're searching backwards (i.e. :search-prev)
         """
-        # :search/:search-next without reverse -> down
-        # :search/:search-next    with reverse -> up
-        # :search-prev         without reverse -> up
-        # :search-prev            with reverse -> down
-        going_up = options['reverse'] ^ prev
+        if not found:
+            message.warning(f"Text '{text}' not found on page!",
+                            replace='find-in-page')
 
-        if found:
-            # Check if the scroll position got smaller and show info.
-            if not going_up and tab.scroller.pos_px().y() < old_scroll_pos.y():
-                message.info("Search hit BOTTOM, continuing at TOP")
-            elif going_up and tab.scroller.pos_px().y() > old_scroll_pos.y():
-                message.info("Search hit TOP, continuing at BOTTOM")
-        else:
-            message.warning("Text '{}' not found on page!".format(text),
-                            replace=True)
+    def _search_navigation_cb(self, result):
+        """Callback called from :search-prev/next."""
+        if result == browsertab.SearchNavigationResult.not_found:
+            self._search_cb(found=False, text=self._tabbed_browser.search_text)
+            return
+        elif result == browsertab.SearchNavigationResult.found:
+            return
+        elif not config.val.search.wrap_messages:
+            return
+
+        messages = {
+            browsertab.SearchNavigationResult.wrap_prevented_bottom:
+                "Search hit BOTTOM",
+            browsertab.SearchNavigationResult.wrap_prevented_top:
+                "Search hit TOP",
+            browsertab.SearchNavigationResult.wrapped_bottom:
+                "Search hit BOTTOM, continuing at TOP",
+            browsertab.SearchNavigationResult.wrapped_top:
+                "Search hit TOP, continuing at BOTTOM",
+        }
+        message.info(messages[result], replace="search-hit-msg")
 
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        maxsplit=0)
@@ -1564,19 +1642,39 @@ class CommandDispatcher:
         options = {
             'ignore_case': config.val.search.ignore_case,
             'reverse': reverse,
-            'wrap': config.val.search.wrap,
         }
 
         self._tabbed_browser.search_text = text
-        self._tabbed_browser.search_options = dict(options)
-
-        cb = functools.partial(self._search_cb, tab=tab,
-                               old_scroll_pos=tab.scroller.pos_px(),
-                               options=options, text=text, prev=False)
-        options['result_cb'] = cb
+        self._tabbed_browser.search_options = options
 
         tab.scroller.before_jump_requested.emit()
-        tab.search.search(text, **options)
+
+        cb = functools.partial(self._search_cb, text=text)
+        tab.search.search(text, **options, result_cb=cb)
+
+    def _search_prev_next(self, count, tab, method):
+        """Continue the search to the prev/next term."""
+        window_text = self._tabbed_browser.search_text
+        window_options = self._tabbed_browser.search_options
+
+        if window_text is None:
+            raise cmdutils.CommandError("No search done yet.")
+
+        tab.scroller.before_jump_requested.emit()
+
+        if window_text is not None and window_text != tab.search.text:
+            tab.search.clear()
+            tab.search.search(window_text, **window_options)
+            count -= 1
+
+        if count == 0:
+            return
+
+        wrap = config.val.search.wrap
+
+        for _ in range(count - 1):
+            method(wrap=wrap)
+        method(callback=self._search_navigation_cb, wrap=wrap)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('count', value=cmdutils.Value.count)
@@ -1587,30 +1685,7 @@ class CommandDispatcher:
             count: How many elements to ignore.
         """
         tab = self._current_widget()
-        window_text = self._tabbed_browser.search_text
-        window_options = self._tabbed_browser.search_options
-
-        if window_text is None:
-            raise cmdutils.CommandError("No search done yet.")
-
-        tab.scroller.before_jump_requested.emit()
-
-        if window_text is not None and window_text != tab.search.text:
-            tab.search.clear()
-            tab.search.search(window_text, **window_options)
-            count -= 1
-
-        if count == 0:
-            return
-
-        cb = functools.partial(self._search_cb, tab=tab,
-                               old_scroll_pos=tab.scroller.pos_px(),
-                               options=window_options, text=window_text,
-                               prev=False)
-
-        for _ in range(count - 1):
-            tab.search.next_result()
-        tab.search.next_result(result_cb=cb)
+        self._search_prev_next(count, tab, tab.search.next_result)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('count', value=cmdutils.Value.count)
@@ -1621,30 +1696,7 @@ class CommandDispatcher:
             count: How many elements to ignore.
         """
         tab = self._current_widget()
-        window_text = self._tabbed_browser.search_text
-        window_options = self._tabbed_browser.search_options
-
-        if window_text is None:
-            raise cmdutils.CommandError("No search done yet.")
-
-        tab.scroller.before_jump_requested.emit()
-
-        if window_text is not None and window_text != tab.search.text:
-            tab.search.clear()
-            tab.search.search(window_text, **window_options)
-            count -= 1
-
-        if count == 0:
-            return
-
-        cb = functools.partial(self._search_cb, tab=tab,
-                               old_scroll_pos=tab.scroller.pos_px(),
-                               options=window_options, text=window_text,
-                               prev=True)
-
-        for _ in range(count - 1):
-            tab.search.prev_result()
-        tab.search.prev_result(result_cb=cb)
+        self._search_prev_next(count, tab, tab.search.prev_result)
 
     def _jseval_cb(self, out):
         """Show the data returned from JS."""
@@ -1739,8 +1791,8 @@ class CommandDispatcher:
             raise cmdutils.CommandError(str(e))
 
         for keyinfo in sequence:
-            press_event = keyinfo.to_event(QEvent.KeyPress)
-            release_event = keyinfo.to_event(QEvent.KeyRelease)
+            press_event = keyinfo.to_event(QEvent.Type.KeyPress)
+            release_event = keyinfo.to_event(QEvent.Type.KeyRelease)
 
             if global_:
                 window = QApplication.focusWindow()
@@ -1847,9 +1899,9 @@ class CommandDispatcher:
         if not window.isFullScreen():
             window.state_before_fullscreen = window.windowState()
         if enter:
-            window.setWindowState(window.windowState() | Qt.WindowFullScreen)
+            window.setWindowState(window.windowState() | Qt.WindowState.WindowFullScreen)
         else:
-            window.setWindowState(window.windowState() ^ Qt.WindowFullScreen)
+            window.setWindowState(window.windowState() ^ Qt.WindowState.WindowFullScreen)
 
         log.misc.debug('state before fullscreen: {}'.format(
             debug.qflags_key(Qt, window.state_before_fullscreen)))
