@@ -1,29 +1,14 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """The ListView to display downloads in."""
 
 import functools
 from typing import Callable, MutableSequence, Tuple, Union
 
-from PyQt5.QtCore import pyqtSlot, QSize, Qt
-from PyQt5.QtWidgets import QListView, QSizePolicy, QMenu, QStyleFactory
+from qutebrowser.qt.core import pyqtSlot, QSize, Qt
+from qutebrowser.qt.widgets import QListView, QSizePolicy, QMenu, QStyleFactory
 
 from qutebrowser.browser import downloads
 from qutebrowser.config import stylesheet
@@ -50,6 +35,7 @@ class DownloadView(QListView):
         QListView {
             background-color: {{ conf.colors.downloads.bar.bg }};
             font: {{ conf.fonts.downloads }};
+            border: 0;
         }
 
         QListView::item {
@@ -62,11 +48,11 @@ class DownloadView(QListView):
         if not utils.is_mac:
             self.setStyle(QStyleFactory.create('Fusion'))
         stylesheet.set_register(self)
-        self.setResizeMode(QListView.Adjust)
-        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
-        self.setFocusPolicy(Qt.NoFocus)
-        self.setFlow(QListView.LeftToRight)
+        self.setResizeMode(QListView.ResizeMode.Adjust)
+        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+        self.setSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Fixed)
+        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
+        self.setFlow(QListView.Flow.LeftToRight)
         self.setSpacing(1)
         self._menu = None
         model.rowsInserted.connect(self._update_geometry)
@@ -74,18 +60,28 @@ class DownloadView(QListView):
         model.dataChanged.connect(self._update_geometry)
         self.setModel(model)
         self.setWrapping(True)
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(self.show_context_menu)
         self.clicked.connect(self.on_clicked)
 
     def __repr__(self):
-        model = self.model()
+        model = qtutils.add_optional(self.model())
+        count: Union[int, str]
         if model is None:
-            count = 'None'  # type: ignore[unreachable]
+            count = 'None'
         else:
             count = model.rowCount()
         return utils.get_repr(self, count=count)
 
+    def _model(self) -> downloads.DownloadModel:
+        """Get the current download model.
+
+        Ensures the model is not None.
+        """
+        model = self.model()
+        assert isinstance(model, downloads.DownloadModel), model
+        return model
+
     @pyqtSlot()
     def _update_geometry(self):
         """Wrapper to call updateGeometry.
@@ -112,7 +108,7 @@ class DownloadView(QListView):
         """
         if not index.isValid():
             return
-        item = self.model().data(index, downloads.ModelRole.item)
+        item = self._model().data(index, downloads.ModelRole.item)
         if item.done and item.successful:
             item.open_file()
             item.remove()
@@ -126,7 +122,7 @@ class DownloadView(QListView):
         Args:
             item: The DownloadItem to get the actions for, or None.
         """
-        model = self.model()
+        model = self._model()
         actions: _ActionListType = []
         if item is None:
             pass
@@ -140,8 +136,10 @@ class DownloadView(QListView):
             actions.append(("Remove", item.remove))
         else:
             actions.append(("Cancel", item.cancel))
-        actions.append(("Copy URL", functools.partial(
-            utils.set_clipboard, item.url().toDisplayString())))
+
+        if item is not None:
+            actions.append(("Copy URL", functools.partial(
+                utils.set_clipboard, item.url().toDisplayString())))
         if model.can_clear():
             actions.append((None, None))
             actions.append(("Remove all finished", model.download_clear))
@@ -152,7 +150,7 @@ class DownloadView(QListView):
         """Show the context menu."""
         index = self.indexAt(point)
         if index.isValid():
-            item = self.model().data(index, downloads.ModelRole.item)
+            item = self._model().data(index, downloads.ModelRole.item)
         else:
             item = None
         self._menu = QMenu(self)
@@ -164,9 +162,12 @@ class DownloadView(QListView):
                 assert name is not None
                 assert handler is not None
                 action = self._menu.addAction(name)
+                assert action is not None
                 action.triggered.connect(handler)
         if actions:
-            self._menu.popup(self.viewport().mapToGlobal(point))
+            viewport = self.viewport()
+            assert viewport is not None
+            self._menu.popup(viewport.mapToGlobal(point))
 
     def minimumSizeHint(self):
         """Override minimumSizeHint so the size is correct in a layout."""
@@ -174,7 +175,7 @@ class DownloadView(QListView):
 
     def sizeHint(self):
         """Return sizeHint based on the view contents."""
-        idx = self.model().last_index()
+        idx = self._model().last_index()
         bottom = self.visualRect(idx).bottom()
         if bottom != -1:
             margins = self.contentsMargins()
