@@ -1,29 +1,14 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Handling of proxies."""
 
-from PyQt5.QtCore import QUrl, pyqtSlot
-from PyQt5.QtNetwork import QNetworkProxy, QNetworkProxyFactory
+from qutebrowser.qt.core import QUrl, pyqtSlot
+from qutebrowser.qt.network import QNetworkProxy, QNetworkProxyFactory
 
 from qutebrowser.config import config, configtypes
-from qutebrowser.utils import message, usertypes, urlutils, utils
+from qutebrowser.utils import message, usertypes, urlutils, utils, qtutils
 from qutebrowser.misc import objects
 from qutebrowser.browser.network import pac
 
@@ -53,7 +38,7 @@ def _warn_for_pac():
 @pyqtSlot()
 def shutdown():
     QNetworkProxyFactory.setApplicationProxyFactory(
-        None)  # type: ignore[arg-type]
+        qtutils.QT_NONE)
 
 
 class ProxyFactory(QNetworkProxyFactory):
@@ -73,11 +58,11 @@ class ProxyFactory(QNetworkProxyFactory):
             return None
 
     def _set_capabilities(self, proxy):
-        if proxy.type() == QNetworkProxy.NoProxy:
+        if proxy.type() == QNetworkProxy.ProxyType.NoProxy:
             return
 
         capabilities = proxy.capabilities()
-        lookup_cap = QNetworkProxy.HostNameLookupCapability
+        lookup_cap = QNetworkProxy.Capability.HostNameLookupCapability
         if config.val.content.proxy_dns_requests:
             capabilities |= lookup_cap
         else:
@@ -97,7 +82,7 @@ class ProxyFactory(QNetworkProxyFactory):
         if proxy is configtypes.SYSTEM_PROXY:
             # On Linux, use "export http_proxy=socks5://host:port" to manually
             # set system proxy.
-            # ref. http://doc.qt.io/qt-5/qnetworkproxyfactory.html#systemProxyForQuery
+            # ref. https://doc.qt.io/qt-6/qnetworkproxyfactory.html#systemProxyForQuery
             proxies = QNetworkProxyFactory.systemProxyForQuery(query)
         elif isinstance(proxy, pac.PACFetcher):
             if objects.backend == usertypes.Backend.QtWebEngine:
