@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Download manager."""
 
@@ -26,12 +11,12 @@ import functools
 import dataclasses
 from typing import Dict, IO, Optional
 
-from PyQt5.QtCore import pyqtSlot, pyqtSignal, QTimer, QUrl
-from PyQt5.QtWidgets import QApplication
-from PyQt5.QtNetwork import QNetworkRequest, QNetworkReply, QNetworkAccessManager
+from qutebrowser.qt.core import pyqtSlot, pyqtSignal, QTimer, QUrl
+from qutebrowser.qt.widgets import QApplication
+from qutebrowser.qt.network import QNetworkRequest, QNetworkReply, QNetworkAccessManager
 
 from qutebrowser.config import config, websettings
-from qutebrowser.utils import message, usertypes, log, urlutils, utils, debug, objreg
+from qutebrowser.utils import message, usertypes, log, urlutils, utils, debug, objreg, qtlog
 from qutebrowser.misc import quitter
 from qutebrowser.browser import downloads
 from qutebrowser.browser.webkit import http
@@ -62,12 +47,8 @@ class DownloadItem(downloads.AbstractDownloadItem):
     As soon as we know the file object, we copy self._buffer over and the next
     readyRead will write to the real file object.
 
-    Class attributes:
-        _MAX_REDIRECTS: The maximum redirection count.
-
     Attributes:
         _retry_info: A _RetryInfo instance.
-        _redirects: How many time we were redirected already.
         _buffer: A BytesIO object to buffer incoming data until we know the
                  target file.
         _read_timer: A Timer which reads the QNetworkReply into self._buffer
@@ -82,7 +63,6 @@ class DownloadItem(downloads.AbstractDownloadItem):
                         arg 0: The new DownloadItem
     """
 
-    _MAX_REDIRECTS = 10
     adopt_download = pyqtSignal(object)  # DownloadItem
 
     def __init__(self, reply, manager):
@@ -102,7 +82,6 @@ class DownloadItem(downloads.AbstractDownloadItem):
         self._read_timer = usertypes.Timer(self, name='download-read-timer')
         self._read_timer.setInterval(500)
         self._read_timer.timeout.connect(self._on_read_timer_timeout)
-        self._redirects = 0
         self._url = reply.url()
         self._init_reply(reply)
 
@@ -110,6 +89,7 @@ class DownloadItem(downloads.AbstractDownloadItem):
         """Create a file object using the internal filename."""
         assert self._filename is not None
         try:
+            # pylint: disable=consider-using-with
             fileobj = open(self._filename, 'wb')
         except OSError as e:
             self._die(e.strerror)
@@ -122,11 +102,13 @@ class DownloadItem(downloads.AbstractDownloadItem):
         if self._reply is None:
             log.downloads.debug("Reply gone while dying")
             return
+
         self._reply.downloadProgress.disconnect()
         self._reply.finished.disconnect()
-        self._reply.error.disconnect()
+        self._reply.errorOccurred.disconnect()
         self._reply.readyRead.disconnect()
-        with log.hide_qt_warning('QNetworkReplyImplPrivate::error: Internal '
+
+        with qtlog.hide_qt_warning('QNetworkReplyImplPrivate::error: Internal '
                                  'problem, this method must only be called '
                                  'once.'):
             # See https://codereview.qt-project.org/#/c/107863/
@@ -134,11 +116,22 @@ class DownloadItem(downloads.AbstractDownloadItem):
         self._reply.deleteLater()
         self._reply = None
         if self.fileobj is not None:
+            pos = self.fileobj.tell()
+            log.downloads.debug(f"File position at error: {pos}")
             try:
                 self.fileobj.close()
             except OSError:
                 log.downloads.exception("Error while closing file object")
 
+            if pos == 0:
+                # Emtpy remaining file
+                filename = self._get_open_filename()
+                log.downloads.debug(f"Removing empty file at {filename}")
+                try:
+                    os.remove(filename)
+                except OSError:
+                    log.downloads.exception("Error while removing empty file")
+
     def _init_reply(self, reply):
         """Set a new reply and connect its signals.
 
@@ -149,11 +142,14 @@ class DownloadItem(downloads.AbstractDownloadItem):
         self.successful = False
         self._reply = reply
         reply.setReadBufferSize(16 * 1024 * 1024)  # 16 MB
+
         reply.downloadProgress.connect(self.stats.on_download_progress)
         reply.finished.connect(self._on_reply_finished)
-        reply.error.connect(self._on_reply_error)
+        reply.errorOccurred.connect(self._on_reply_error)
         reply.readyRead.connect(self._on_ready_read)
         reply.metaDataChanged.connect(self._on_meta_data_changed)
+        reply.redirected.connect(self._on_redirected)
+
         self._retry_info = _RetryInfo(request=reply.request(),
                                       manager=reply.manager())
         if not self.fileobj:
@@ -161,9 +157,16 @@ class DownloadItem(downloads.AbstractDownloadItem):
         # We could have got signals before we connected slots to them.
         # Here no signals are connected to the DownloadItem yet, so we use a
         # singleShot QTimer to emit them after they are connected.
-        if reply.error() != QNetworkReply.NoError:
+        if reply.error() != QNetworkReply.NetworkError.NoError:
             QTimer.singleShot(0, lambda: self._die(reply.errorString()))
 
+    @pyqtSlot(QUrl)
+    def _on_redirected(self, url):
+        if self._reply is None:
+            log.downloads.warning(f"redirected: REPLY GONE -> {url}")
+        else:
+            log.downloads.debug(f"redirected: {self._reply.url()} -> {url}")
+
     def _do_cancel(self):
         self._read_timer.stop()
         if self._reply is not None:
@@ -202,6 +205,17 @@ class DownloadItem(downloads.AbstractDownloadItem):
         # Note: self._reply is deleted when the download finishes
         return self._url
 
+    def origin(self) -> QUrl:
+        if self._reply is None:
+            return QUrl()
+        origin = self._reply.request().originatingObject()
+        try:
+            return origin.url()
+        except AttributeError:
+            # Raised either if origin is None or some object that doesn't
+            # have its own url.
+            return QUrl()
+
     def _ensure_can_set_filename(self, filename):
         if self.fileobj is not None:  # pragma: no cover
             raise ValueError("fileobj was already set! filename: {}, "
@@ -274,7 +288,7 @@ class DownloadItem(downloads.AbstractDownloadItem):
             self.fileobj.write(self._reply.readAll())
         if self._autoclose:
             self.fileobj.close()
-        self.successful = self._reply.error() == QNetworkReply.NoError
+        self.successful = self._reply.error() == QNetworkReply.NetworkError.NoError
         self._reply.close()
         self._reply.deleteLater()
         self._reply = None
@@ -295,9 +309,6 @@ class DownloadItem(downloads.AbstractDownloadItem):
             return
         self._read_timer.stop()
         self.stats.finish()
-        is_redirected = self._handle_redirect()
-        if is_redirected:
-            return
         log.downloads.debug("Reply finished, fileobj {}".format(self.fileobj))
         if self.fileobj is not None:
             # We can do a "delayed" write immediately to empty the buffer and
@@ -322,7 +333,7 @@ class DownloadItem(downloads.AbstractDownloadItem):
     @pyqtSlot('QNetworkReply::NetworkError')
     def _on_reply_error(self, code):
         """Handle QNetworkReply errors."""
-        if code == QNetworkReply.OperationCanceledError:
+        if code == QNetworkReply.NetworkError.OperationCanceledError:
             return
 
         if self._reply is None:
@@ -352,48 +363,6 @@ class DownloadItem(downloads.AbstractDownloadItem):
         for key, value in self._reply.rawHeaderPairs():
             self.raw_headers[bytes(key)] = bytes(value)
 
-    def _handle_redirect(self):
-        """Handle an HTTP redirect.
-
-        Return:
-            True if the download was redirected, False otherwise.
-        """
-        assert self._reply is not None
-        redirect = self._reply.attribute(
-            QNetworkRequest.RedirectionTargetAttribute)
-        if redirect is None or redirect.isEmpty():
-            return False
-        new_url = self._reply.url().resolved(redirect)
-        new_request = self._reply.request()
-        if new_url == new_request.url():
-            return False
-
-        if self._redirects > self._MAX_REDIRECTS:
-            self._die("Maximum redirection count reached!")
-            self.delete()
-            return True  # so on_reply_finished aborts
-
-        log.downloads.debug("{}: Handling redirect".format(self))
-        self._redirects += 1
-        new_request.setUrl(new_url)
-
-        old_reply = self._reply
-        assert old_reply is not None
-        old_reply.finished.disconnect(self._on_reply_finished)
-
-        self._read_timer.stop()
-        self._reply = None
-        if self.fileobj is not None:
-            self.fileobj.seek(0)
-
-        log.downloads.debug("redirected: {} -> {}".format(
-            old_reply.url(), new_request.url()))
-        new_reply = old_reply.manager().get(new_request)
-        self._init_reply(new_reply)
-
-        old_reply.deleteLater()
-        return True
-
     def _uses_nam(self, nam):
         """Check if this download uses the given QNetworkAccessManager."""
         assert self._retry_info is not None
@@ -410,8 +379,17 @@ class DownloadManager(downloads.AbstractDownloadManager):
 
     Attributes:
         _networkmanager: A NetworkManager for generic downloads.
+
+    Class attributes:
+        _MAX_REDIRECTS: The maximum redirection count.
     """
 
+    # Same as many browsers
+    # https://fetch.spec.whatwg.org/#http-redirect-fetch
+    # https://source.chromium.org/chromium/chromium/src/+/main:net/url_request/url_request.h;l=97;drc=3c19a2edb96d3d5b56a7481349a357fdbdf8ecf0
+    # https://stackoverflow.com/questions/9384474/in-chrome-how-many-redirects-are-too-many
+    _MAX_REDIRECTS = 20
+
     def __init__(self, parent=None):
         super().__init__(parent)
         self._networkmanager = networkmanager.NetworkManager(
@@ -435,11 +413,19 @@ class DownloadManager(downloads.AbstractDownloadManager):
             return None
 
         req = QNetworkRequest(url)
-        user_agent = websettings.user_agent(url)
-        req.setHeader(QNetworkRequest.UserAgentHeader, user_agent)
 
+        user_agent = websettings.user_agent(url)
+        req.setHeader(QNetworkRequest.KnownHeaders.UserAgentHeader, user_agent)
         if not cache:
-            req.setAttribute(QNetworkRequest.CacheSaveControlAttribute, False)
+            req.setAttribute(QNetworkRequest.Attribute.CacheSaveControlAttribute, False)
+
+        # Needed for Qt 5, default on Qt 6
+        # We don't set this on the QNAM because QtWebKit handles redirects manually.
+        req.setAttribute(
+            QNetworkRequest.Attribute.RedirectPolicyAttribute,
+            QNetworkRequest.RedirectPolicy.NoLessSafeRedirectPolicy,
+        )
+        req.setMaximumRedirectsAllowed(self._MAX_REDIRECTS)
 
         return self.get_request(req, **kwargs)
 
@@ -491,6 +477,7 @@ class DownloadManager(downloads.AbstractDownloadManager):
         Args:
             request: The QNetworkRequest to download.
             target: Where to save the download as downloads.DownloadTarget.
+            suggested_fn: The filename to use for the file.
             **kwargs: Passed to _fetch_request.
 
         Return:
@@ -498,8 +485,8 @@ class DownloadManager(downloads.AbstractDownloadManager):
         """
         # WORKAROUND for Qt corrupting data loaded from cache:
         # https://bugreports.qt.io/browse/QTBUG-42757
-        request.setAttribute(QNetworkRequest.CacheLoadControlAttribute,
-                             QNetworkRequest.AlwaysNetwork)
+        request.setAttribute(QNetworkRequest.Attribute.CacheLoadControlAttribute,
+                             QNetworkRequest.CacheLoadControl.AlwaysNetwork)
 
         if suggested_fn is None:
             suggested_fn = self._get_suggested_filename(request)
@@ -535,6 +522,9 @@ class DownloadManager(downloads.AbstractDownloadManager):
             target: Where to save the download as downloads.DownloadTarget.
             auto_remove: Whether to remove the download even if
                          downloads.remove_finished is set to -1.
+            suggested_filename: The filename to use for the file.
+            prompt_download_directory: Whether to prompt for a location to
+                                       download the file to.
 
         Return:
             The created DownloadItem.
@@ -549,6 +539,9 @@ class DownloadManager(downloads.AbstractDownloadManager):
         download = DownloadItem(reply, manager=self)
         self._init_item(download, auto_remove, suggested_filename)
 
+        if download.cancel_for_origin():
+            return download
+
         if target is not None:
             download.set_target(target)
             return download
@@ -582,6 +575,7 @@ class DownloadManager(downloads.AbstractDownloadManager):
         """
         assert nam.adopted_downloads == 0
         for download in self.downloads:
+            assert isinstance(download, DownloadItem), download
             if download._uses_nam(nam):  # pylint: disable=protected-access
                 nam.adopt_download(download)
         return nam.adopted_downloads
