@@ -1,39 +1,30 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Generic web element related code."""
 
-import typing
+from typing import Iterator, Optional, Set, TYPE_CHECKING, Union, Dict
 import collections.abc
 
-from PyQt5.QtCore import QUrl, Qt, QEvent, QTimer, QRect, QPoint
-from PyQt5.QtGui import QMouseEvent
+from qutebrowser.qt import machinery
+from qutebrowser.qt.core import QUrl, Qt, QEvent, QTimer, QRect, QPointF
+from qutebrowser.qt.gui import QMouseEvent
 
 from qutebrowser.config import config
 from qutebrowser.keyinput import modeman
 from qutebrowser.utils import log, usertypes, utils, qtutils, objreg
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from qutebrowser.browser import browsertab
 
 
-JsValueType = typing.Union[int, float, str, None]
+JsValueType = Union[int, float, str, None]
+
+if machinery.IS_QT6:
+    KeybordModifierType = Qt.KeyboardModifier
+else:
+    KeybordModifierType = Union[Qt.KeyboardModifiers, Qt.KeyboardModifier]
 
 
 class Error(Exception):
@@ -58,7 +49,8 @@ def css_selector(group: str, url: QUrl) -> str:
     return ','.join(selectors[group])
 
 
-class AbstractWebElement(collections.abc.MutableMapping):
+# MutableMapping is only generic in Python 3.9+
+class AbstractWebElement(collections.abc.MutableMapping):  # type: ignore[type-arg]
 
     """A wrapper around QtWebKit/QtWebEngine web element."""
 
@@ -80,7 +72,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
     def __delitem__(self, key: str) -> None:
         raise NotImplementedError
 
-    def __iter__(self) -> typing.Iterator[str]:
+    def __iter__(self) -> Iterator[str]:
         raise NotImplementedError
 
     def __len__(self) -> int:
@@ -88,8 +80,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
 
     def __repr__(self) -> str:
         try:
-            html = utils.compact_text(
-                self.outer_xml(), 500)  # type: typing.Optional[str]
+            html: Optional[str] = utils.compact_text(self.outer_xml(), 500)
         except Error:
             html = None
         return utils.get_repr(self, html=html)
@@ -102,7 +93,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
         """Get the geometry for this element."""
         raise NotImplementedError
 
-    def classes(self) -> typing.Set[str]:
+    def classes(self) -> Set[str]:
         """Get a set of classes assigned to this element."""
         raise NotImplementedError
 
@@ -132,6 +123,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
         """Dispatch an event to the element.
 
         Args:
+            event: The name of the event.
             bubbles: Whether this event should bubble.
             cancelable: Whether this event can be cancelled.
             composed: Whether the event will trigger listeners outside of a
@@ -160,9 +152,6 @@ class AbstractWebElement(collections.abc.MutableMapping):
     def is_content_editable(self) -> bool:
         """Check if an element has a contenteditable attribute.
 
-        Args:
-            elem: The QWebElement to check.
-
         Return:
             True if the element has a contenteditable attribute,
             False otherwise.
@@ -233,6 +222,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
             'span': ['cm-'],  # Jupyter Notebook
         }
         relevant_classes = classes[self.tag_name()]
+        # pylint: disable=consider-using-any-or-all
         for klass in self.classes():
             if any(klass.strip().startswith(e) for e in relevant_classes):
                 return True
@@ -282,7 +272,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
         """Remove target from link."""
         raise NotImplementedError
 
-    def resolve_url(self, baseurl: QUrl) -> typing.Optional[QUrl]:
+    def resolve_url(self, baseurl: QUrl) -> Optional[QUrl]:
         """Resolve the URL in the element's src/href attribute.
 
         Args:
@@ -318,7 +308,7 @@ class AbstractWebElement(collections.abc.MutableMapping):
         """Return True if clicking this element needs user interaction."""
         raise NotImplementedError
 
-    def _mouse_pos(self) -> QPoint:
+    def _mouse_pos(self) -> QPointF:
         """Get the position to click/hover."""
         # Click the center of the largest square fitting into the top/left
         # corner of the rectangle, this will help if part of the <a> element
@@ -332,41 +322,37 @@ class AbstractWebElement(collections.abc.MutableMapping):
         pos = rect.center()
         if pos.x() < 0 or pos.y() < 0:
             raise Error("Element position is out of view!")
-        return pos
+        return QPointF(pos)
 
     def _move_text_cursor(self) -> None:
         """Move cursor to end after clicking."""
         raise NotImplementedError
 
     def _click_fake_event(self, click_target: usertypes.ClickTarget,
-                          button: Qt.MouseButton = Qt.LeftButton) -> None:
+                          button: Qt.MouseButton = Qt.MouseButton.LeftButton) -> None:
         """Send a fake click event to the element."""
         pos = self._mouse_pos()
 
         log.webelem.debug("Sending fake click to {!r} at position {} with "
                           "target {}".format(self, pos, click_target))
 
-        target_modifiers = {
-            usertypes.ClickTarget.normal: Qt.NoModifier,
-            usertypes.ClickTarget.window: Qt.AltModifier | Qt.ShiftModifier,
-            usertypes.ClickTarget.tab: Qt.ControlModifier,
-            usertypes.ClickTarget.tab_bg: Qt.ControlModifier,
+        target_modifiers: Dict[usertypes.ClickTarget, KeybordModifierType] = {
+            usertypes.ClickTarget.normal: Qt.KeyboardModifier.NoModifier,
+            usertypes.ClickTarget.window: Qt.KeyboardModifier.AltModifier | Qt.KeyboardModifier.ShiftModifier,
+            usertypes.ClickTarget.tab: Qt.KeyboardModifier.ControlModifier,
+            usertypes.ClickTarget.tab_bg: Qt.KeyboardModifier.ControlModifier,
         }
         if config.val.tabs.background:
-            target_modifiers[usertypes.ClickTarget.tab] |= Qt.ShiftModifier
+            target_modifiers[usertypes.ClickTarget.tab] |= Qt.KeyboardModifier.ShiftModifier
         else:
-            target_modifiers[usertypes.ClickTarget.tab_bg] |= Qt.ShiftModifier
+            target_modifiers[usertypes.ClickTarget.tab_bg] |= Qt.KeyboardModifier.ShiftModifier
 
-        modifiers = typing.cast(Qt.KeyboardModifiers,
-                                target_modifiers[click_target])
+        modifiers = target_modifiers[click_target]
 
         events = [
-            QMouseEvent(QEvent.MouseMove, pos, Qt.NoButton, Qt.NoButton,
-                        Qt.NoModifier),
-            QMouseEvent(QEvent.MouseButtonPress, pos, button, button,
-                        modifiers),
-            QMouseEvent(QEvent.MouseButtonRelease, pos, button, Qt.NoButton,
-                        modifiers),
+            QMouseEvent(QEvent.Type.MouseMove, pos, Qt.MouseButton.NoButton, Qt.MouseButton.NoButton, Qt.KeyboardModifier.NoModifier),
+            QMouseEvent(QEvent.Type.MouseButtonPress, pos, button, button, modifiers),
+            QMouseEvent(QEvent.Type.MouseButtonRelease, pos, button, Qt.MouseButton.NoButton, modifiers),
         ]
 
         for evt in events:
@@ -404,8 +390,8 @@ class AbstractWebElement(collections.abc.MutableMapping):
         elif click_target == usertypes.ClickTarget.window:
             from qutebrowser.mainwindow import mainwindow
             window = mainwindow.MainWindow(private=tabbed_browser.is_private)
-            window.show()
             window.tabbed_browser.tabopen(url)
+            window.show()
         else:
             raise ValueError("Unknown ClickTarget {}".format(click_target))
 
@@ -450,11 +436,11 @@ class AbstractWebElement(collections.abc.MutableMapping):
     def hover(self) -> None:
         """Simulate a mouse hover over the element."""
         pos = self._mouse_pos()
-        event = QMouseEvent(QEvent.MouseMove, pos, Qt.NoButton, Qt.NoButton,
-                            Qt.NoModifier)
+        event = QMouseEvent(QEvent.Type.MouseMove, pos, Qt.MouseButton.NoButton, Qt.MouseButton.NoButton,
+                            Qt.KeyboardModifier.NoModifier)
         self._tab.send_event(event)
 
     def right_click(self) -> None:
         """Simulate a right-click on the element."""
         self._click_fake_event(usertypes.ClickTarget.normal,
-                               button=Qt.RightButton)
+                               button=Qt.MouseButton.RightButton)
