@@ -1,47 +1,32 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Configuration storage and config-related utilities."""
 
 import copy
 import contextlib
 import functools
-import typing
-from typing import Any
+from typing import (TYPE_CHECKING, Any, Callable, Dict, Iterator, List, Mapping,
+                    MutableMapping, MutableSequence, Optional, Tuple, cast)
 
-from PyQt5.QtCore import pyqtSignal, QObject, QUrl
+from qutebrowser.qt.core import pyqtSignal, QObject, QUrl
 
+from qutebrowser.commands import cmdexc, parser
 from qutebrowser.config import configdata, configexc, configutils
 from qutebrowser.utils import utils, log, urlmatch
 from qutebrowser.misc import objects
 from qutebrowser.keyinput import keyutils
 
-if typing.TYPE_CHECKING:
-    from typing import Tuple, MutableMapping
+if TYPE_CHECKING:
     from qutebrowser.config import configcache, configfiles
     from qutebrowser.misc import savemanager
 
 # An easy way to access the config from other code via config.val.foo
-val = typing.cast('ConfigContainer', None)
-instance = typing.cast('Config', None)
-key_instance = typing.cast('KeyConfig', None)
-cache = typing.cast('configcache.ConfigCache', None)
+val = cast('ConfigContainer', None)
+instance = cast('Config', None)
+key_instance = cast('KeyConfig', None)
+cache = cast('configcache.ConfigCache', None)
 
 # Keeping track of all change filters to validate them later.
 change_filters = []
@@ -84,7 +69,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
                 not configdata.is_valid_prefix(self._option)):
             raise configexc.NoOptionError(self._option)
 
-    def check_match(self, option: typing.Optional[str]) -> bool:
+    def check_match(self, option: Optional[str]) -> bool:
         """Check if the given option matches the filter."""
         if option is None:
             # Called directly, not from a config change event.
@@ -97,7 +82,10 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         else:
             return False
 
-    def __call__(self, func: typing.Callable) -> typing.Callable:
+    def __call__(
+        self,
+        func: Callable[..., None],
+    ) -> Callable[..., None]:
         """Filter calls to the decorated function.
 
         Gets called when a function should be decorated.
@@ -105,7 +93,9 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         Adds a filter which returns if we're not interested in the change-event
         and calls the wrapped function if we are.
 
-        We assume the function passed doesn't take any parameters.
+        We assume the function passed doesn't take any parameters. However, it
+        could take a "self" argument, so we can't clearly express this in the
+        type above.
 
         Args:
             func: The function to be decorated.
@@ -115,7 +105,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         """
         if self._function:
             @functools.wraps(func)
-            def func_wrapper(option: str = None) -> typing.Any:
+            def func_wrapper(option: str = None) -> Any:
                 """Call the underlying function."""
                 if self.check_match(option):
                     return func()
@@ -123,8 +113,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
             return func_wrapper
         else:
             @functools.wraps(func)
-            def meth_wrapper(wrapper_self: typing.Any,
-                             option: str = None) -> typing.Any:
+            def meth_wrapper(wrapper_self: Any, option: str = None) -> Any:
                 """Call the underlying function."""
                 if self.check_match(option):
                     return func(wrapper_self)
@@ -142,7 +131,7 @@ class KeyConfig:
         _config: The Config object to be used.
     """
 
-    _ReverseBindings = typing.Dict[str, typing.MutableSequence[str]]
+    _ReverseBindings = Dict[str, MutableSequence[str]]
 
     def __init__(self, config: 'Config') -> None:
         self._config = config
@@ -154,10 +143,7 @@ class KeyConfig:
         if mode not in configdata.DATA['bindings.default'].default:
             raise configexc.KeybindingError("Invalid mode {}!".format(mode))
 
-    def get_bindings_for(
-            self,
-            mode: str
-    ) -> typing.Dict[keyutils.KeySequence, str]:
+    def get_bindings_for(self, mode: str) -> Dict[keyutils.KeySequence, str]:
         """Get the combined bindings for the given mode."""
         bindings = dict(val.bindings.default[mode])
         for key, binding in val.bindings.commands[mode].items():
@@ -167,13 +153,40 @@ class KeyConfig:
                 bindings[key] = binding
         return bindings
 
+    def _implied_cmd(self, cmdline: str) -> Optional[str]:
+        """Return cmdline, or the implied cmd if cmdline is a cmd-set-text."""
+        try:
+            results = parser.CommandParser().parse_all(cmdline)
+        except cmdexc.NoSuchCommandError:
+            return None
+
+        result = results[0]
+        if result.cmd.name not in ["set-cmd-text", "cmd-set-text"]:
+            return cmdline
+        if not result.args:
+            return None  # doesn't look like this sets a command
+        *flags, cmd = result.args
+        if "-a" in flags or "--append" in flags or not cmd.startswith(":"):
+            return None  # doesn't look like this sets a command
+        return cmd.lstrip(":")
+
     def get_reverse_bindings_for(self, mode: str) -> '_ReverseBindings':
-        """Get a dict of commands to a list of bindings for the mode."""
-        cmd_to_keys = {}  # type: KeyConfig._ReverseBindings
+        """Get a dict of commands to a list of bindings for the mode.
+
+        This is intended for user-facing display of keybindings.
+        As such, bindings for 'set-cmd-text [flags] :<cmd> ...' are translated
+        to '<cmd> ...', as from the user's perspective these keys behave like
+        bindings for '<cmd>' (that allow for further input before running).
+
+        See #5942.
+        """
+        cmd_to_keys: KeyConfig._ReverseBindings = {}
         bindings = self.get_bindings_for(mode)
         for seq, full_cmd in sorted(bindings.items()):
-            for cmd in full_cmd.split(';;'):
-                cmd = cmd.strip()
+            for cmdtext in full_cmd.split(';;'):
+                cmd = self._implied_cmd(cmdtext.strip())
+                if not cmd:
+                    continue
                 cmd_to_keys.setdefault(cmd, [])
                 # Put bindings involving modifiers last
                 if any(info.modifiers for info in seq):
@@ -185,7 +198,7 @@ class KeyConfig:
     def get_command(self,
                     key: keyutils.KeySequence,
                     mode: str,
-                    default: bool = False) -> typing.Optional[str]:
+                    default: bool = False) -> Optional[str]:
         """Get the command for a given key (or None)."""
         self._validate(key, mode)
         if default:
@@ -278,7 +291,7 @@ class Config(QObject):
                  yaml_config: 'configfiles.YamlConfig',
                  parent: QObject = None) -> None:
         super().__init__(parent)
-        self._mutables = {}  # type: MutableMapping[str, Tuple[Any, Any]]
+        self._mutables: MutableMapping[str, Tuple[Any, Any]] = {}
         self._yaml = yaml_config
         self._init_values()
         self.yaml_loaded = False
@@ -286,11 +299,11 @@ class Config(QObject):
 
     def _init_values(self) -> None:
         """Populate the self._values dict."""
-        self._values = {}  # type: typing.Mapping
+        self._values: Mapping[str, configutils.Values] = {}
         for name, opt in configdata.DATA.items():
             self._values[name] = configutils.Values(opt)
 
-    def __iter__(self) -> typing.Iterator[configutils.Values]:
+    def __iter__(self) -> Iterator[configutils.Values]:
         """Iterate over configutils.Values items."""
         yield from self._values.values()
 
@@ -345,7 +358,11 @@ class Config(QObject):
             deleted = name in configdata.MIGRATIONS.deleted
             renamed = configdata.MIGRATIONS.renamed.get(name)
             exception = configexc.NoOptionError(
-                name, deleted=deleted, renamed=renamed)
+                name,
+                deleted=deleted,
+                renamed=renamed,
+                all_names=list(configdata.DATA),
+            )
             raise exception from None
 
     def ensure_has_opt(self, name: str) -> None:
@@ -359,6 +376,8 @@ class Config(QObject):
         """Get the given setting converted for Python code.
 
         Args:
+            name: The name of the setting to get.
+            url: The URL to get the value for.
             fallback: Use the global value if there's no URL-specific one.
         """
         opt = self.get_opt(name)
@@ -391,7 +410,7 @@ class Config(QObject):
 
     def get_obj_for_pattern(
             self, name: str, *,
-            pattern: typing.Optional[urlmatch.UrlPattern]
+            pattern: Optional[urlmatch.UrlPattern]
     ) -> Any:
         """Get the given setting as object (for YAML/config.py).
 
@@ -479,8 +498,12 @@ class Config(QObject):
 
     def unset(self, name: str, *,
               save_yaml: bool = False,
-              pattern: urlmatch.UrlPattern = None) -> None:
-        """Set the given setting back to its default."""
+              pattern: urlmatch.UrlPattern = None) -> bool:
+        """Set the given setting back to its default.
+
+        Return:
+            True if there was a change, False if nothing changed.
+        """
         opt = self.get_opt(name)
         self._check_yaml(opt, save_yaml)
         changed = self._values[name].remove(pattern)
@@ -490,6 +513,8 @@ class Config(QObject):
         if save_yaml:
             self._yaml.unset(name, pattern=pattern)
 
+        return changed
+
     def clear(self, *, save_yaml: bool = False) -> None:
         """Clear all settings in the config.
 
@@ -513,21 +538,25 @@ class Config(QObject):
         Here, we check all those saved copies for mutations, and if something
         mutated, we call set_obj again so we save the new value.
         """
-        for name, (old_value, new_value) in self._mutables.items():
+        mutables = self._mutables.items()
+        self._mutables = {}
+        for name, (old_value, new_value) in mutables:
             if old_value != new_value:
                 log.config.debug("{} was mutated, updating".format(name))
                 self.set_obj(name, new_value, save_yaml=save_yaml)
-        self._mutables = {}
 
-    def dump_userconfig(self) -> str:
+    def dump_userconfig(self, *, include_hidden: bool = False) -> str:
         """Get the part of the config which was changed by the user.
 
+        Args:
+            include_hidden: Include default scoped configs.
+
         Return:
             The changed config part as string.
         """
-        lines = []  # type: typing.List[str]
+        lines: List[str] = []
         for values in sorted(self, key=lambda v: v.opt.name):
-            lines += values.dump()
+            lines += values.dump(include_hidden=include_hidden)
 
         if not lines:
             return '<Default configuration>'
@@ -564,15 +593,24 @@ class ConfigContainer:
                               pattern=self._pattern)
 
     @contextlib.contextmanager
-    def _handle_error(self, action: str, name: str) -> typing.Iterator[None]:
+    def _handle_error(self, action: str) -> Iterator[None]:
         try:
             yield
         except configexc.Error as e:
             if self._configapi is None:
                 raise
-            text = "While {} '{}'".format(action, name)
+            text = f"While {action}"
             self._configapi.errors.append(configexc.ConfigErrorDesc(text, e))
 
+    def _with_prefix(self, prefix: str) -> 'ConfigContainer':
+        """Get a new ConfigContainer for the given prefix."""
+        return ConfigContainer(
+            config=self._config,
+            configapi=self._configapi,
+            pattern=self._pattern,
+            prefix=prefix,
+        )
+
     def __getattr__(self, attr: str) -> Any:
         """Get an option or a new ConfigContainer with the added prefix.
 
@@ -587,11 +625,9 @@ class ConfigContainer:
 
         name = self._join(attr)
         if configdata.is_valid_prefix(name):
-            return ConfigContainer(config=self._config,
-                                   configapi=self._configapi,
-                                   prefix=name, pattern=self._pattern)
+            return self._with_prefix(name)
 
-        with self._handle_error('getting', name):
+        with self._handle_error(f"getting '{name}'"):
             if self._configapi is None:
                 # access from Python code
                 return self._config.get(name)
@@ -600,6 +636,13 @@ class ConfigContainer:
                 return self._config.get_mutable_obj(
                     name, pattern=self._pattern)
 
+        # If we arrived here, there was an error while getting the config option. Most
+        # likely, someone did something like "c.content.host_blocking.lists" but
+        # "c.content.host_blocking" doesn't actually exist. To avoid an AttributeError
+        # which leads to a confusing error message, return another ConfigContainer so
+        # that the chain can keep going.
+        return self._with_prefix(name)  # type: ignore[unreachable]
+
     def __setattr__(self, attr: str, value: Any) -> None:
         """Set the given option in the config."""
         if attr.startswith('_'):
@@ -607,12 +650,11 @@ class ConfigContainer:
             return
 
         name = self._join(attr)
-        with self._handle_error('setting', name):
+        with self._handle_error(f"setting '{name}'"):
             self._config.set_obj(name, value, pattern=self._pattern)
 
     def _join(self, attr: str) -> str:
         """Get the prefix joined with the given attribute."""
         if self._prefix:
             return '{}.{}'.format(self._prefix, attr)
-        else:
-            return attr
+        return attr
