@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2017-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Initialization of the configuration."""
 
@@ -23,7 +8,7 @@ import argparse
 import os.path
 import sys
 
-from PyQt5.QtWidgets import QMessageBox
+from qutebrowser.qt.widgets import QMessageBox
 
 from qutebrowser.api import config as configapi
 from qutebrowser.config import (config, configdata, configfiles, configtypes,
@@ -58,11 +43,18 @@ def early_init(args: argparse.Namespace) -> None:
     objreg.register('config-commands', config_commands, command_only=True)
 
     config_file = standarddir.config_py()
+    custom_config_py = args.config_py is not None
+
     global _init_errors
 
     try:
-        if os.path.exists(config_file):
-            configfiles.read_config_py(config_file)
+        if os.path.exists(config_file) or custom_config_py:
+            # If we have a custom --config-py flag, we want it to be fatal if it doesn't
+            # exist, so we don't silently fall back to autoconfig.yml in that case.
+            configfiles.read_config_py(
+                config_file,
+                warn_autoconfig=not custom_config_py,
+            )
         else:
             configfiles.read_autoconfig()
     except configexc.ConfigFileErrors as e:
@@ -128,9 +120,9 @@ def late_init(save_manager: savemanager.SaveManager) -> None:
         errbox = msgbox.msgbox(parent=None,
                                title="Error while reading config",
                                text=_init_errors.to_html(),
-                               icon=QMessageBox.Warning,
+                               icon=QMessageBox.Icon.Warning,
                                plain_text=False)
-        errbox.exec_()
+        errbox.exec()
 
         if _init_errors.fatal:
             sys.exit(usertypes.Exit.err_init)
