@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Management of sessions - saved tabs/windows."""
 
@@ -23,24 +8,24 @@ import os
 import os.path
 import itertools
 import urllib
-import typing
-import glob
 import shutil
+import pathlib
+from typing import Any, Iterable, MutableMapping, MutableSequence, Optional, Union, cast
 
-from PyQt5.QtCore import Qt, QUrl, QObject, QPoint, QTimer, QDateTime
-from PyQt5.QtWidgets import QApplication
+from qutebrowser.qt.core import Qt, QUrl, QObject, QPoint, QTimer, QDateTime
 import yaml
 
 from qutebrowser.utils import (standarddir, objreg, qtutils, log, message,
-                               utils)
+                               utils, usertypes)
 from qutebrowser.api import cmdutils
 from qutebrowser.config import config, configfiles
 from qutebrowser.completion.models import miscmodels
 from qutebrowser.mainwindow import mainwindow
 from qutebrowser.qt import sip
+from qutebrowser.misc import objects, throttle
 
 
-_JsonType = typing.MutableMapping[str, typing.Any]
+_JsonType = MutableMapping[str, Any]
 
 
 class Sentinel:
@@ -49,9 +34,9 @@ class Sentinel:
 
 
 default = Sentinel()
-session_manager = typing.cast('SessionManager', None)
+session_manager = cast('SessionManager', None)
 
-ArgType = typing.Union[str, Sentinel]
+ArgType = Union[str, Sentinel]
 
 
 def init(parent=None):
@@ -60,30 +45,27 @@ def init(parent=None):
     Args:
         parent: The parent to use for the SessionManager.
     """
-    base_path = os.path.join(standarddir.data(), 'sessions')
+    base_path = pathlib.Path(standarddir.data()) / 'sessions'
 
     # WORKAROUND for https://github.com/qutebrowser/qutebrowser/issues/5359
-    backup_path = os.path.join(base_path, 'before-qt-515')
-    if (os.path.exists(base_path) and
-            not os.path.exists(backup_path) and
-            qtutils.version_check('5.15', compiled=False)):
-        os.mkdir(backup_path)
-        for filename in glob.glob(os.path.join(base_path, '*.yml')):
-            shutil.copy(filename, backup_path)
+    backup_path = base_path / 'before-qt-515'
+    do_backup = objects.backend == usertypes.Backend.QtWebEngine
 
-    try:
-        os.mkdir(base_path)
-    except FileExistsError:
-        pass
+    if base_path.exists() and not backup_path.exists() and do_backup:
+        backup_path.mkdir()
+        for path in base_path.glob('*.yml'):
+            shutil.copy(path, backup_path)
+
+    base_path.mkdir(exist_ok=True)
 
     global session_manager
-    session_manager = SessionManager(base_path, parent)
+    session_manager = SessionManager(str(base_path), parent)
 
 
-def shutdown(session: typing.Optional[ArgType], last_window: bool) -> None:
+def shutdown(session: Optional[ArgType], last_window: bool) -> None:
     """Handle a shutdown by saving sessions and removing the autosave file."""
     if session_manager is None:
-        return  # type: ignore
+        return  # type: ignore[unreachable]
 
     try:
         if session is not None:
@@ -153,10 +135,12 @@ class SessionManager(QObject):
 
     def __init__(self, base_path, parent=None):
         super().__init__(parent)
-        self.current = None  # type: typing.Optional[str]
+        self.current: Optional[str] = None
         self._base_path = base_path
         self._last_window_session = None
         self.did_load = False
+        # throttle autosaves to one minute apart
+        self.save_autosave = throttle.Throttle(self._save_autosave, 60 * 1000)
 
     def _get_session_path(self, name, check_exists=False):
         """Get the session path based on a session name or absolute path.
@@ -196,18 +180,17 @@ class SessionManager(QObject):
         Return:
             A dict with the saved data for this item.
         """
-        data = {
+        data: _JsonType = {
             'url': bytes(item.url().toEncoded()).decode('ascii'),
-        }  # type: _JsonType
+        }
 
         if item.title():
             data['title'] = item.title()
-        else:
+        elif tab.history.current_idx() == idx:
             # https://github.com/qutebrowser/qutebrowser/issues/879
-            if tab.history.current_idx() == idx:
-                data['title'] = tab.title()
-            else:
-                data['title'] = data['url']
+            data['title'] = tab.title()
+        else:
+            data['title'] = data['url']
 
         if item.originalUrl() != item.url():
             encoded = item.originalUrl().toEncoded()
@@ -222,7 +205,7 @@ class SessionManager(QObject):
             # QtWebEngine
             user_data = None
 
-        data['last_visited'] = item.lastVisited().toString(Qt.ISODate)
+        data['last_visited'] = item.lastVisited().toString(Qt.DateFormat.ISODate)
 
         if tab.history.current_idx() == idx:
             pos = tab.scroller.pos_px()
@@ -239,19 +222,30 @@ class SessionManager(QObject):
 
         return data
 
-    def _save_tab(self, tab, active):
+    def _save_tab(self, tab, active, with_history=True):
         """Get a dict with data for a single tab.
 
         Args:
             tab: The WebView to save.
             active: Whether the tab is currently active.
+            with_history: Include the tab's history.
         """
-        data = {'history': []}  # type: _JsonType
+        data: _JsonType = {'history': []}
         if active:
             data['active'] = True
-        for idx, item in enumerate(tab.history):
+
+        history = tab.history if with_history else [tab.history.current_item()]
+
+        for idx, item in enumerate(history):
             qtutils.ensure_valid(item)
             item_data = self._save_tab_item(tab, idx, item)
+
+            if not item.url().isValid():
+                # WORKAROUND Qt 6.5 regression
+                # https://github.com/qutebrowser/qutebrowser/issues/7696
+                log.sessions.debug(f"Skipping invalid history item: {item}")
+                continue
+
             if item.url().scheme() == 'qute' and item.url().host() == 'back':
                 # don't add qute://back to the session file
                 if item_data.get('active', False) and data['history']:
@@ -261,11 +255,11 @@ class SessionManager(QObject):
                 data['history'].append(item_data)
         return data
 
-    def _save_all(self, *, only_window=None, with_private=False):
+    def _save_all(self, *, only_window=None, with_private=False, with_history=True):
         """Get a dict with data for all windows/tabs."""
-        data = {'windows': []}  # type: _JsonType
+        data: _JsonType = {'windows': []}
         if only_window is not None:
-            winlist = [only_window]  # type: typing.Iterable[int]
+            winlist: Iterable[int] = [only_window]
         else:
             winlist = objreg.window_registry
 
@@ -282,8 +276,8 @@ class SessionManager(QObject):
             if tabbed_browser.is_private and not with_private:
                 continue
 
-            win_data = {}  # type: _JsonType
-            active_window = QApplication.instance().activeWindow()
+            win_data: _JsonType = {}
+            active_window = objects.qapp.activeWindow()
             if getattr(active_window, 'win_id', None) == win_id:
                 win_data['active'] = True
             win_data['geometry'] = bytes(main_window.saveGeometry())
@@ -292,7 +286,8 @@ class SessionManager(QObject):
                 win_data['private'] = True
             for i, tab in enumerate(tabbed_browser.widgets()):
                 active = i == tabbed_browser.widget.currentIndex()
-                win_data['tabs'].append(self._save_tab(tab, active))
+                win_data['tabs'].append(self._save_tab(tab, active,
+                                                       with_history=with_history))
             data['windows'].append(win_data)
         return data
 
@@ -313,7 +308,7 @@ class SessionManager(QObject):
         return name
 
     def save(self, name, last_window=False, load_next_time=False,
-             only_window=None, with_private=False):
+             only_window=None, with_private=False, with_history=True):
         """Save a named session.
 
         Args:
@@ -324,6 +319,7 @@ class SessionManager(QObject):
             load_next_time: If set, prepares this session to be load next time.
             only_window: If set, only tabs in the specified window is saved.
             with_private: Include private windows.
+            with_history: Include tab history.
 
         Return:
             The name of the saved session.
@@ -339,7 +335,8 @@ class SessionManager(QObject):
                 return None
         else:
             data = self._save_all(only_window=only_window,
-                                  with_private=with_private)
+                                  with_private=with_private,
+                                  with_history=with_history)
         log.sessions.vdebug(  # type: ignore[attr-defined]
             "Saving data: {}".format(data))
         try:
@@ -352,7 +349,7 @@ class SessionManager(QObject):
             configfiles.state['general']['session'] = name
         return name
 
-    def save_autosave(self):
+    def _save_autosave(self):
         """Save the autosave session."""
         try:
             self.save('_autosave')
@@ -361,6 +358,8 @@ class SessionManager(QObject):
 
     def delete_autosave(self):
         """Delete the autosave session."""
+        # cancel any in-flight saves
+        self.save_autosave.cancel()
         try:
             self.delete('_autosave')
         except SessionNotFoundError:
@@ -377,7 +376,7 @@ class SessionManager(QObject):
     def _load_tab(self, new_tab, data):  # noqa: C901
         """Load yaml data into a newly opened tab."""
         entries = []
-        lazy_load = []  # type: typing.MutableSequence[_JsonType]
+        lazy_load: MutableSequence[_JsonType] = []
         # use len(data['history'])
         # -> dropwhile empty if not session.lazy_session
         lazy_index = len(data['history'])
@@ -436,10 +435,10 @@ class SessionManager(QObject):
                 orig_url = url
 
             if histentry.get("last_visited"):
-                last_visited = QDateTime.fromString(
+                last_visited: Optional[QDateTime] = QDateTime.fromString(
                     histentry.get("last_visited"),
-                    Qt.ISODate,
-                )  # type: typing.Optional[QDateTime]
+                    Qt.DateFormat.ISODate,
+                )
             else:
                 last_visited = None
 
@@ -460,7 +459,6 @@ class SessionManager(QObject):
         """Turn yaml data into windows."""
         window = mainwindow.MainWindow(geometry=win['geometry'],
                                        private=win.get('private', None))
-        window.show()
         tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                     window=window.win_id)
         tab_to_focus = None
@@ -473,6 +471,8 @@ class SessionManager(QObject):
                 new_tab.set_pinned(True)
         if tab_to_focus is not None:
             tabbed_browser.widget.setCurrentIndex(tab_to_focus)
+
+        window.show()
         if win.get('active', False):
             QTimer.singleShot(0, tabbed_browser.widget.activateWindow)
 
@@ -552,31 +552,31 @@ def session_load(name: str, *,
     except SessionError as e:
         raise cmdutils.CommandError("Error while loading session: {}"
                                     .format(e))
-    else:
-        if clear:
-            for win in old_windows:
-                win.close()
-        if delete:
-            try:
-                session_manager.delete(name)
-            except SessionError as e:
-                log.sessions.exception("Error while deleting session!")
-                raise cmdutils.CommandError("Error while deleting session: {}"
-                                            .format(e))
-            else:
-                log.sessions.debug("Loaded & deleted session {}.".format(name))
+    if clear:
+        for win in old_windows:
+            win.close()
+    if delete:
+        try:
+            session_manager.delete(name)
+        except SessionError as e:
+            log.sessions.exception("Error while deleting session!")
+            raise cmdutils.CommandError("Error while deleting session: {}"
+                                        .format(e))
+        log.sessions.debug("Loaded & deleted session {}.".format(name))
 
 
 @cmdutils.register()
 @cmdutils.argument('name', completion=miscmodels.session)
 @cmdutils.argument('win_id', value=cmdutils.Value.win_id)
 @cmdutils.argument('with_private', flag='p')
+@cmdutils.argument('no_history', flag='n')
 def session_save(name: ArgType = default, *,
                  current: bool = False,
                  quiet: bool = False,
                  force: bool = False,
                  only_active_window: bool = False,
                  with_private: bool = False,
+                 no_history: bool = False,
                  win_id: int = None) -> None:
     """Save a session.
 
@@ -588,6 +588,7 @@ def session_save(name: ArgType = default, *,
         force: Force saving internal sessions (starting with an underline).
         only_active_window: Saves only tabs of the currently active window.
         with_private: Include private windows.
+        no_history: Don't store tab history.
     """
     if not isinstance(name, Sentinel) and name.startswith('_') and not force:
         raise cmdutils.CommandError("{} is an internal session, use --force "
@@ -600,16 +601,17 @@ def session_save(name: ArgType = default, *,
     try:
         if only_active_window:
             name = session_manager.save(name, only_window=win_id,
-                                        with_private=True)
+                                        with_private=True,
+                                        with_history=not no_history)
         else:
-            name = session_manager.save(name, with_private=with_private)
+            name = session_manager.save(name, with_private=with_private,
+                                        with_history=not no_history)
     except SessionError as e:
         raise cmdutils.CommandError("Error while saving session: {}".format(e))
+    if quiet:
+        log.sessions.debug("Saved session {}.".format(name))
     else:
-        if quiet:
-            log.sessions.debug("Saved session {}.".format(name))
-        else:
-            message.info("Saved session {}.".format(name))
+        message.info("Saved session {}.".format(name))
 
 
 @cmdutils.register()
@@ -632,8 +634,7 @@ def session_delete(name: str, *, force: bool = False) -> None:
         log.sessions.exception("Error while deleting session!")
         raise cmdutils.CommandError("Error while deleting session: {}"
                                     .format(e))
-    else:
-        log.sessions.debug("Deleted session {}.".format(name))
+    log.sessions.debug("Deleted session {}.".format(name))
 
 
 def load_default(name):
