@@ -1,48 +1,26 @@
 #!/usr/bin/env python3
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
-# Copyright 2014-2018 Claude (longneck) <longneck@scratchbook.ch>
-
-# This file is part of qutebrowser.
+# SPDX-FileCopyrightText: Claude (longneck) <longneck@scratchbook.ch>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 
 """Tool to import data from other browsers.
 
-Currently importing bookmarks from Netscape Bookmark files and Mozilla
-profiles is supported.
+Currently importing bookmarks from Netscape HTML Bookmark files, Chrome
+profiles, and Mozilla profiles is supported.
 """
 
 
 import argparse
+import textwrap
 import sqlite3
 import os
 import urllib.parse
 import json
 import string
 
-browser_default_input_format = {
-    'chromium': 'chrome',
-    'chrome': 'chrome',
-    'ie': 'netscape',
-    'firefox': 'mozilla',
-    'seamonkey': 'mozilla',
-    'palemoon': 'mozilla',
-}
-
 
 def main():
     args = get_args()
@@ -68,15 +46,9 @@ def main():
         bookmark_types = ['bookmark', 'keyword']
     if not output_format:
         output_format = 'quickmark'
-    if not input_format:
-        if args.browser:
-            input_format = browser_default_input_format[args.browser]
-        else:
-            #default to netscape
-            input_format = 'netscape'
 
     import_function = {
-        'netscape': import_netscape_bookmarks,
+        'html': import_html_bookmarks,
         'mozilla': import_moz_places,
         'chrome': import_chrome,
     }
@@ -87,20 +59,25 @@ def main():
 def get_args():
     """Get the argparse parser."""
     parser = argparse.ArgumentParser(
-        epilog="To import bookmarks from Chromium, Firefox or IE, "
-        "export them to HTML in your browsers bookmark manager. ")
-    parser.add_argument(
-        'browser',
-        help="Which browser? {%(choices)s}",
-        choices=browser_default_input_format.keys(),
-        nargs='?',
-        metavar='browser')
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+        epilog=textwrap.dedent('''
+            To import bookmarks, you'll need the path to your profile or an
+            exported HTML file from your browser's bookmark manager. Redirect
+            the output from this script to the appropriate file in your
+            qutebrowser config directory (listed in the output of :version),
+            usually done with the '>' operator; for example,
+                ./importer.py -i mozilla your_profile_path > ~/.config/qutebrowser/quickmarks
+
+            Common browsers with native input format support:
+                chrome: Chrome, Chromium, Edge
+                mozilla: Firefox, SeaMonkey, Pale Moon
+        '''))
     parser.add_argument(
         '-i',
         '--input-format',
-        help='Which input format? (overrides browser default; "netscape" if '
-        'neither given)',
-        choices=set(browser_default_input_format.values()),
+        help="Which input format? Defaults to html",
+        choices=['html', 'mozilla', 'chrome'],
+        default='html',
         required=False)
     parser.add_argument(
         '-b',
@@ -186,7 +163,7 @@ def opensearch_convert(url):
     return search_escape(url.format(**subst)).replace('%s', '{}')
 
 
-def import_netscape_bookmarks(bookmarks_file, bookmark_types, output_format):
+def import_html_bookmarks(bookmarks_file, bookmark_types, output_format):
     """Import bookmarks from a NETSCAPE-Bookmark-file v1.
 
     Generated by Chromium, Firefox, IE and possibly more browsers. Not all
