@@ -1,22 +1,8 @@
 #!/usr/bin/env python3
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2020-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
-
-# This file is part of qutebrowser.
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 
 """Create a local virtualenv with a PyQt install."""
@@ -30,13 +16,22 @@ import os.path
 import shutil
 import venv as pyvenv
 import subprocess
-from typing import List, Optional, Tuple, Dict, Union
+import platform
+from typing import List, Tuple, Dict, Union
 
 sys.path.insert(0, os.path.join(os.path.dirname(__file__), os.pardir))
 from scripts import utils, link_pyqt
 
 
 REPO_ROOT = pathlib.Path(__file__).parent.parent
+# for --only-binary / --no-binary
+PYQT_PACKAGES = [
+    "PyQt5",
+    "PyQtWebEngine",
+
+    "PyQt6",
+    "PyQt6-WebEngine",
+]
 
 
 class Error(Exception):
@@ -77,13 +72,12 @@ def parse_args(argv: List[str] = None) -> argparse.Namespace:
     parser.add_argument('--pyqt-wheels-dir',
                         default='wheels',
                         help="Directory to get PyQt wheels from.")
+    parser.add_argument('--pyqt-snapshot',
+                        help="Comma-separated list to install from the Riverbank "
+                        "PyQt snapshot server")
     parser.add_argument('--virtualenv',
                         action='store_true',
                         help="Use virtualenv instead of venv.")
-    parser.add_argument('--asciidoc', help="Full path to python and "
-                        "asciidoc.py. If not given, it's searched in PATH.",
-                        nargs=2, required=False,
-                        metavar=('PYTHON', 'ASCIIDOC'))
     parser.add_argument('--dev',
                         action='store_true',
                         help="Also install dev/test dependencies.")
@@ -99,6 +93,18 @@ def parse_args(argv: List[str] = None) -> argparse.Namespace:
     return parser.parse_args(argv)
 
 
+def _version_key(v):
+    """Sort PyQt requirement file prefixes.
+
+    If we have a filename like requirements-pyqt-pyinstaller.txt, that should
+    always be sorted after all others (hence we return a "999" key).
+    """
+    try:
+        return tuple(int(v) for c in v.split('.'))
+    except ValueError:
+        return (999,)
+
+
 def pyqt_versions() -> List[str]:
     """Get a list of all available PyQt versions.
 
@@ -110,11 +116,15 @@ def pyqt_versions() -> List[str]:
     for req in requirements_dir.glob('requirements-pyqt-*.txt'):
         version_set.add(req.stem.split('-')[-1])
 
-    versions = sorted(version_set,
-                      key=lambda v: [int(c) for c in v.split('.')])
+    versions = sorted(version_set, key=_version_key)
     return versions + ['auto']
 
 
+def _is_qt6_version(version: str) -> bool:
+    """Check if the given version is Qt 6."""
+    return version in ["auto", "6"] or version.startswith("6.")
+
+
 def run_venv(
         venv_dir: pathlib.Path,
         executable,
@@ -136,7 +146,7 @@ def run_venv(
         return subprocess.run(
             [str(venv_dir / subdir / executable)] + [str(arg) for arg in args],
             check=True,
-            universal_newlines=capture_output or capture_error,
+            text=capture_output or capture_error,
             stdout=subprocess.PIPE if capture_output else None,
             stderr=subprocess.PIPE if capture_error else None,
             env=proc_env,
@@ -169,7 +179,7 @@ def delete_old_venv(venv_dir: pathlib.Path) -> None:
                     'remove it.'.format(venv_dir))
 
     print_command('rm -r', venv_dir, venv=False)
-    shutil.rmtree(str(venv_dir))
+    shutil.rmtree(venv_dir)
 
 
 def create_venv(venv_dir: pathlib.Path, use_virtualenv: bool = False) -> None:
@@ -205,7 +215,7 @@ def requirements_file(name: str) -> pathlib.Path:
 
 def pyqt_requirements_file(version: str) -> pathlib.Path:
     """Get the filename of the requirements file for the given PyQt version."""
-    name = 'pyqt' if version == 'auto' else 'pyqt-{}'.format(version)
+    name = 'pyqt-6' if version == 'auto' else f'pyqt-{version}'
     return requirements_file(name)
 
 
@@ -214,22 +224,48 @@ def install_pyqt_binary(venv_dir: pathlib.Path, version: str) -> None:
     utils.print_title("Installing PyQt from binary")
     utils.print_col("No proprietary codec support will be available in "
                     "qutebrowser.", 'bold')
+
+    if _is_qt6_version(version):
+        supported_archs = {
+            'linux': {'x86_64'},
+            'win32': {'AMD64'},
+            'darwin': {'x86_64', 'arm64'},
+        }
+    else:
+        supported_archs = {
+            'linux': {'x86_64'},
+            'win32': {'x86', 'AMD64'},
+            'darwin': {'x86_64'},
+        }
+
+    if sys.platform not in supported_archs:
+        utils.print_error(f"{sys.platform} is not a supported platform by PyQt binary "
+                          "packages, this will most likely fail.")
+    elif platform.machine() not in supported_archs[sys.platform]:
+        utils.print_error(
+            f"{platform.machine()} is not a supported architecture for PyQt binaries "
+            f"on {sys.platform}, this will most likely fail.")
+    elif sys.platform == 'linux' and platform.libc_ver()[0] != 'glibc':
+        utils.print_error("Non-glibc Linux is not a supported platform for PyQt "
+                          "binaries, this will most likely fail.")
+
     pip_install(venv_dir, '-r', pyqt_requirements_file(version),
-                '--only-binary', 'PyQt5,PyQtWebEngine')
+                '--only-binary', ','.join(PYQT_PACKAGES))
 
 
 def install_pyqt_source(venv_dir: pathlib.Path, version: str) -> None:
     """Install PyQt from the source tarball."""
     utils.print_title("Installing PyQt from sources")
     pip_install(venv_dir, '-r', pyqt_requirements_file(version),
-                '--verbose', '--no-binary', 'PyQt5,PyQtWebEngine')
+                '--verbose', '--no-binary', ','.join(PYQT_PACKAGES))
 
 
-def install_pyqt_link(venv_dir: pathlib.Path) -> None:
+def install_pyqt_link(venv_dir: pathlib.Path, version: str) -> None:
     """Install PyQt by linking a system-wide install."""
     utils.print_title("Linking system-wide PyQt")
     lib_path = link_pyqt.get_venv_lib_path(str(venv_dir))
-    link_pyqt.link_pyqt(sys.executable, lib_path)
+    major_version: str = "6" if _is_qt6_version(version) else "5"
+    link_pyqt.link_pyqt(sys.executable, lib_path, version=major_version)
 
 
 def install_pyqt_wheels(venv_dir: pathlib.Path,
@@ -240,6 +276,13 @@ def install_pyqt_wheels(venv_dir: pathlib.Path,
     pip_install(venv_dir, *wheels)
 
 
+def install_pyqt_snapshot(venv_dir: pathlib.Path, packages: List[str]) -> None:
+    """Install PyQt packages from the snapshot server."""
+    utils.print_title("Installing PyQt snapshots")
+    pip_install(venv_dir, '-U', *packages, '--no-deps', '--pre',
+                '--index-url', 'https://riverbankcomputing.com/pypi/simple/')
+
+
 def apply_xcb_util_workaround(
         venv_dir: pathlib.Path,
         pyqt_type: str,
@@ -257,11 +300,17 @@ def apply_xcb_util_workaround(
     if pyqt_type != 'binary':
         print("Workaround not needed: Not installing from PyQt binaries.")
         return
-    if pyqt_version not in ['auto', '5.15']:
+    if _is_qt6_version(pyqt_version):
         print("Workaround not needed: Not installing Qt 5.15.")
         return
 
-    libs = _find_libs()
+    try:
+        libs = _find_libs()
+    except subprocess.CalledProcessError as e:
+        utils.print_error(
+            f'Workaround failed: ldconfig exited with status {e.returncode}')
+        return
+
     abi_type = 'libc6,x86-64'  # the only one PyQt wheels are available for
 
     if ('libxcb-util.so.1', abi_type) in libs:
@@ -319,7 +368,7 @@ def _find_libs() -> Dict[Tuple[str, str], List[str]]:
     for line in ldconfig_proc.stdout.splitlines():
         match = pattern.fullmatch(line.strip())
         if match is None:
-            if 'libs found in cache' not in line:
+            if 'libs found in cache' not in line and 'Cache generated by:' not in line:
                 utils.print_col(f'Failed to match ldconfig output: {line}', 'yellow')
             continue
 
@@ -332,13 +381,17 @@ def _find_libs() -> Dict[Tuple[str, str], List[str]]:
     return all_libs
 
 
-def run_qt_smoke_test(venv_dir: pathlib.Path) -> None:
+def run_qt_smoke_test_single(
+    venv_dir: pathlib.Path, *,
+    debug: bool,
+    pyqt_version: str,
+) -> None:
     """Make sure the Qt installation works."""
     utils.print_title("Running Qt smoke test")
     code = [
         'import sys',
-        'from PyQt5.QtWidgets import QApplication',
-        'from PyQt5.QtCore import qVersion, QT_VERSION_STR, PYQT_VERSION_STR',
+        'from qutebrowser.qt.widgets import QApplication',
+        'from qutebrowser.qt.core import qVersion, QT_VERSION_STR, PYQT_VERSION_STR',
         'print(f"Python: {sys.version}")',
         'print(f"qVersion: {qVersion()}")',
         'print(f"QT_VERSION_STR: {QT_VERSION_STR}")',
@@ -347,20 +400,45 @@ def run_qt_smoke_test(venv_dir: pathlib.Path) -> None:
         'print("Qt seems to work properly!")',
         'print()',
     ]
+    env = {
+        'QUTE_QT_WRAPPER': 'PyQt6' if _is_qt6_version(pyqt_version) else 'PyQt5',
+    }
+    if debug:
+        env['QT_DEBUG_PLUGINS'] = '1'
+
     try:
         run_venv(
             venv_dir,
             'python', '-c', '; '.join(code),
-            env={'QT_DEBUG_PLUGINS': '1'},
+            env=env,
             capture_error=True
         )
     except Error as e:
         proc_e = e.__cause__
         assert isinstance(proc_e, subprocess.CalledProcessError), proc_e
         print(proc_e.stderr)
-        raise Error(
-            f"Smoke test failed with status {proc_e.returncode}. "
-            "You might find additional information in the debug output above.")
+
+        msg = f"Smoke test failed with status {proc_e.returncode}."
+        if debug:
+            msg += " You might find additional information in the debug output above."
+        raise Error(msg)
+
+
+def run_qt_smoke_test(venv_dir: pathlib.Path, *, pyqt_version: str) -> None:
+    """Make sure the Qt installation works."""
+    # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-104415
+    no_debug = pyqt_version == "6.3" and sys.platform == "darwin"
+    if no_debug:
+        try:
+            run_qt_smoke_test_single(venv_dir, debug=False, pyqt_version=pyqt_version)
+        except Error as e:
+            print(e)
+            print("Rerunning with debug output...")
+            print("NOTE: This will likely segfault due to a Qt bug:")
+            print("https://bugreports.qt.io/browse/QTBUG-104415")
+            run_qt_smoke_test_single(venv_dir, debug=True, pyqt_version=pyqt_version)
+    else:
+        run_qt_smoke_test_single(venv_dir, debug=True, pyqt_version=pyqt_version)
 
 
 def install_requirements(venv_dir: pathlib.Path) -> None:
@@ -375,7 +453,13 @@ def install_dev_requirements(venv_dir: pathlib.Path) -> None:
     utils.print_title("Installing dev dependencies")
     pip_install(venv_dir,
                 '-r', str(requirements_file('dev')),
-                '-r', requirements_file('tests'))
+                '-r', str(requirements_file('check-manifest')),
+                '-r', str(requirements_file('flake8')),
+                '-r', str(requirements_file('mypy')),
+                '-r', str(requirements_file('pyroma')),
+                '-r', str(requirements_file('vulture')),
+                '-r', str(requirements_file('yamllint')),
+                '-r', str(requirements_file('tests')))
 
 
 def install_qutebrowser(venv_dir: pathlib.Path) -> None:
@@ -384,18 +468,14 @@ def install_qutebrowser(venv_dir: pathlib.Path) -> None:
     pip_install(venv_dir, '-e', str(REPO_ROOT))
 
 
-def regenerate_docs(venv_dir: pathlib.Path,
-                    asciidoc: Optional[Tuple[str, str]]):
+def regenerate_docs(venv_dir: pathlib.Path):
     """Regenerate docs using asciidoc."""
     utils.print_title("Generating documentation")
-    if asciidoc is not None:
-        a2h_args = ['--asciidoc'] + asciidoc
-    else:
-        a2h_args = []
-    script_path = pathlib.Path(__file__).parent / 'asciidoc2html.py'
+    pip_install(venv_dir, '-r', str(requirements_file('docs')))
 
-    print_command('python3 scripts/asciidoc2html.py', *a2h_args, venv=True)
-    run_venv(venv_dir, 'python', str(script_path), *a2h_args)
+    script_path = pathlib.Path(__file__).parent / 'asciidoc2html.py'
+    print_command('python3 scripts/asciidoc2html.py', venv=True)
+    run_venv(venv_dir, 'python', str(script_path))
 
 
 def update_repo():
@@ -411,12 +491,17 @@ def install_pyqt(venv_dir, args):
     """Install PyQt in the virtualenv."""
     if args.pyqt_type == 'binary':
         install_pyqt_binary(venv_dir, args.pyqt_version)
+        if args.pyqt_snapshot:
+            install_pyqt_snapshot(venv_dir, args.pyqt_snapshot.split(','))
     elif args.pyqt_type == 'source':
         install_pyqt_source(venv_dir, args.pyqt_version)
     elif args.pyqt_type == 'link':
-        install_pyqt_link(venv_dir)
+        install_pyqt_link(venv_dir, args.pyqt_version)
     elif args.pyqt_type == 'wheels':
         wheels_dir = pathlib.Path(args.pyqt_wheels_dir)
+        if not wheels_dir.is_dir():
+            raise Error(
+                f"Wheels directory {wheels_dir} doesn't exist or is not a directory")
         install_pyqt_wheels(venv_dir, wheels_dir)
     elif args.pyqt_type == 'skip':
         pass
@@ -429,14 +514,18 @@ def run(args) -> None:
     venv_dir = pathlib.Path(args.venv_dir)
     utils.change_cwd()
 
-    if (args.pyqt_version != 'auto' and
-            args.pyqt_type not in ['binary', 'source']):
-        raise Error('The --pyqt-version option is only available when installing PyQt '
-                    'from binary or source')
+    if args.pyqt_version != 'auto' and args.pyqt_type == 'skip':
+        raise Error('Cannot use --pyqt-version with --pyqt-type skip')
+    if args.pyqt_type == 'link' and args.pyqt_version not in ['auto', '5', '6']:
+        raise Error('Invalid --pyqt-version {args.pyqt_version}, only 5 or 6 '
+                    'permitted with --pyqt-type=link')
 
     if args.pyqt_wheels_dir != 'wheels' and args.pyqt_type != 'wheels':
         raise Error('The --pyqt-wheels-dir option is only available when installing '
                     'PyQt from wheels')
+    if args.pyqt_snapshot and args.pyqt_type != 'binary':
+        raise Error('The --pyqt-snapshot option is only available when installing '
+                    'PyQt from binaries')
 
     if args.update:
         utils.print_title("Updating repository")
@@ -451,16 +540,16 @@ def run(args) -> None:
     install_pyqt(venv_dir, args)
 
     apply_xcb_util_workaround(venv_dir, args.pyqt_type, args.pyqt_version)
-    if args.pyqt_type != 'skip' and not args.skip_smoke_test:
-        run_qt_smoke_test(venv_dir)
 
     install_requirements(venv_dir)
     install_qutebrowser(venv_dir)
     if args.dev:
         install_dev_requirements(venv_dir)
 
+    if args.pyqt_type != 'skip' and not args.skip_smoke_test:
+        run_qt_smoke_test(venv_dir, pyqt_version=args.pyqt_version)
     if not args.skip_docs:
-        regenerate_docs(venv_dir, args.asciidoc)
+        regenerate_docs(venv_dir)
 
 
 def main():
